name = 'wrapt_timeout_decorator'
title = 'there are many timeout decorators out there - that one focuses on correctness when using with Classes, methods, class methods, static methods and so on'
version = '1.4.0'
url = 'https://github.com/bitranox/wrapt_timeout_decorator'
author = 'Robert Nowotny'
author_email = 'bitranox@gmail.com'
shell_command = 'wrapt_timeout_decorator'


def print_version() -> None:
    print('version: 1.4.0')


def print_info() -> None:
    print("""information for "wrapt_timeout_decorator":

          there are many timeout decorators out there - that one focuses on correctness when using with Classes, methods, class methods, static methods and so on

          Version      : 1.4.0
          url          : https://github.com/bitranox/wrapt_timeout_decorator
          author       : Robert Nowotny
          author_email : bitranox@gmail.com""")
